# try:
#     import yaml
# except:
#     raise ImportError('Please `pip install -r requirements.txt`.')
# else:
#     versions = []
#     versions.extend([
#         yaml.__version__ == '5.4.1'
#     ])

#     if not all(versions):
#         raise ImportError('''\
# requirements: yaml==5.4.1\
# ''')
