class DocTypeError(Exception): pass

class KeywordAttrsError(Exception): pass

class MoreNodesFound(Exception): pass

class NoNodesFound(Exception): pass

class TextNotFound(Exception): pass

class UpdateError(Exception): pass

class AddNodeError(Exception): pass

class PopError(Exception): pass

class ZipParseError(Exception): pass

class AnalysisTypeError(Exception): pass

class UnparseDictError(Exception): pass

class ExcelCellLocError(Exception): pass

class MoveError(Exception): pass

class EnvironmentError(Exception): pass
